/////////////////////////////////////////////////////////////////////////////// 
// Copyright (C) 2003-2011, Open Design Alliance (the "Alliance"). 
// All rights reserved. 
// 
// This software and its documentation and related materials are owned by 
// the Alliance. The software may only be incorporated into application 
// programs owned by members of the Alliance, subject to a signed 
// Membership Agreement and Supplemental Software License Agreement with the
// Alliance. The structure and organization of this software are the valuable  
// trade secrets of the Alliance and its suppliers. The software is also 
// protected by copyright law and international treaty provisions. Application  
// programs incorporating this software must include the following statement 
// with their copyright notices:
//   
//   This application incorporates Teigha(R) software pursuant to a license 
//   agreement with Open Design Alliance.
//   Teigha(R) for .dwg files Copyright (C) 2003-2011 by Open Design Alliance. 
//   All rights reserved.
//
// By use of this software, its documentation or related materials, you 
// acknowledge and accept the above terms.
//
//
// *DWG is the native and proprietary file format for AutoCAD(R) and a trademark  
// of Autodesk, Inc.  Open Design Alliance is not associated with Autodesk.
///////////////////////////////////////////////////////////////////////////////

#ifndef DbAssocAction_INCLUDED_
#define DbAssocAction_INCLUDED_ /* {Secret} */

#include "DbConstraintsExports.h"
#include "DbAssocGlobal.h"
#include "DbEntity.h"     // Because of DragStat enum

class OdDbImpAssocAction;

#include "TD_PackPush.h"


/** Description:
  Library: Source *code* provided.
  {group:Contraint_Classes} 
*/
class DBCONSTRAINTS_EXPORT OdDbAssocAction : public OdDbObject
{
public:     
    ODRX_DECLARE_MEMBERS(OdDbAssocAction);

    OdDbAssocAction(bool createImpObj = true);
    ~OdDbAssocAction();

    OdDbObjectId actionBody() const;
    
    OdResult setActionBody(const OdDbObjectId& actionBodyId);

    OdDbAssocStatus status() const;

    OdResult setStatus(OdDbAssocStatus newStatus, 
                       bool notifyOwningNetwork = true,
                       bool setInOwnedActions   = false);

    OdDbObjectId owningNetwork() const;

    OdResult setOwningNetwork(const OdDbObjectId& networkId, bool alsoSetAsDatabaseOwner);


    OdResult getDependencies(bool readDependenciesWanted,
                             bool writeDependenciesWanted,
                             OdDbObjectIdArray& dependencyIds) const;

    OdResult addDependency(const OdDbObjectId& dependencyId, 
                           bool setThisActionAsOwningAction = true);

    OdResult removeDependency(const OdDbObjectId& dependencyId, 
                              bool alsoEraseIt);

    OdResult removeAllDependencies(bool alsoEraseThem);

    OdResult getDependentObjects(bool readDependenciesWanted,
                                 bool writeDependenciesWanted,
                                 OdDbObjectIdArray& objectIds) const;
#if 0
    bool isOwnedDependency(const OdDbAssocDependency* pDependency) const;

    bool isExternalDependency(const OdDbAssocDependency* pDependency) const;
#endif
    bool isRelevantDependencyChange(const OdDbAssocDependency* pDependency) const;
#if 0
    bool hasDependencyCachedValue(const OdDbAssocDependency* pDependency) const;

    bool areDependenciesOnTheSameThing(const OdDbAssocDependency* pDependency1, 
                                       const OdDbAssocDependency* pDependency2) const;

    bool areDependenciesEqual(const OdDbAssocDependency* pDependency1, 
                              const OdDbAssocDependency* pDependency2) const;

    static OdResult getActionsDependentOnObject(const OdDbObject* pObject,
                                                bool readDependenciesWanted,
                                                bool writeDependenciesWanted,                                   
                                                OdDbObjectIdArray& actionIds);
#endif
    void dependentObjectCloned(const OdDbAssocDependency* pDependency, 
                               const OdDbObject*          pDbObj, 
                               const OdDbObject*          pNewObj);
#if 0
    OdResult addMoreObjectsToDeepClone(OdDbIdMapping& idMap, 
                                       OdDbObjectIdArray& additionalObjectsToClone) const;

#endif

    OdResult postProcessAfterDeepClone(OdDbIdMapping& idMap); 

    OdResult postProcessAfterDeepCloneCancel(OdDbIdMapping& idMap);

    bool isActionEvaluationInProgress() const;

    OdDbAssocEvaluationCallback* currentEvaluationCallback() const;

    OdResult evaluateDependencies();

    void evaluateDependency(OdDbAssocDependency* pDependency);

    OdResult ownedDependencyStatusChanged(OdDbAssocDependency* pOwnedDependency, 
                                          OdDbAssocStatus      previousStatus);
#if 0
    OdResult transformActionBy(const OdGeMatrix3d& transform);

    bool isEqualTo(const OdDbAssocAction* pOtherAction) const;

    OdDbAssocEvaluationPriority evaluationPriority() const;
#endif
    void getDependentActionsToEvaluate(OdDbActionsToEvaluateCallback* pActionsToEvaluateCallback) const;

    void evaluate(OdDbAssocEvaluationCallback* pEvaluationCallback);

    OdDbObjectId objectThatOwnsNetworkInstance() const;
#if 0
    void dragStatus(const OdDb::DragStat status);
#endif

  OdResult dwgInFields(OdDbDwgFiler* pFiler);
  OdResult dwgOutFields(OdDbDwgFiler* pFiler) const;

  OdResult dxfInFields(OdDbDxfFiler* pFiler);
  OdResult dxfOutFields(OdDbDxfFiler* pFiler) const;

  OdResult setActionIndex(int newIndex);

// AC2013
// OdDbAssocActionParams stuff
  OdResult removeAllParams(bool bEraseThem);
  int paramCount() const;
  const OdDbObjectIdArray& ownedParams() const;
  OdResult addParam(const OdDbObjectId& paramId, int& paramIndex);
#if 0// TODO - can be impleneted after OdDbAssocActionParam class implementation
  OdResult addParam(const OdString& paramName, OdRxClass* pParamClass, OdDbObjectId& paramId, int& paramIndex);
#endif
  OdResult removeParam(const OdDbObjectId& paramId, bool alsoEraseIt);
#if 0// TODO - can be impleneted after OdDbAssocActionParam class implementation
  const OdDbObjectIdArray& paramsAtName(const OdString& paramName) const;
  OdDbObjectId paramAtName(const OdString& paramName, int index = 0) const;
#endif
  OdDbObjectId paramAtIndex(int paramIndex) const;
#if 0// TODO - can be impleneted after OdDbAssocActionParam class implementation
  void ownedValueParamNames(OdStringArray& paramNames) const;
  OdResult getValueParamArray(const OdString& paramName,
                                        OdArray<OdDbEvalVariant>& values,
                                        OdStringArray&        expressions,
                                        OdStringArray&        evaluatorIds) const;
  OdResult getValueParam(const OdString&  paramName,
                               OdDbEvalVariant& value,
                               OdString&        expression,
                               OdString&        evaluatorId,
                               int              valueIndex = 0) const;

  OdResult setValueParamArray(const OdString&                 paramName,
                              const OdArray<OdDbEvalVariant>& values,
                              const OdStringArray&            expressions,
                              const OdStringArray&            evaluatorIds,
                              OdStringArray&                  errorMessages,
                              bool                            silentMode);
  OdResult setValueParam(const OdString&        paramName,
                         const OdDbEvalVariant& value,
                         const OdString&        expression,
                         const OdString&        evaluatorId,
                         OdString&              errorMessage,
                         bool                   silentMode,
                         int                    valueIndex = 0);
  OdValue::UnitType valueParamUnitType(const OdString& paramName) const;
  OdResult setValueParamUnitType(const OdString& paramName, OdValue::UnitType unitType);
  OdResult removeValueParam(const OdString& paramName);
  OdResult valueParamInputVariables(const OdString& paramName, OdDbObjectIdArray& variableIds) const;
  OdResult setValueParamControlledObjectDep(const OdString& paramName, const OdDbObjectId& controlledObjectDepId);
  OdResult updateValueParamControlledObject(const OdString& paramName) const;
  OdResult updateValueParamFromControlledObject(const OdString& paramName);
  OdResult updateAllObjectsControlledByValueParams() const;
  OdResult transformAllConstantGeometryParams(const OdGeMatrix3d& transform);
  OdResult scaleAllDistanceValueParams(double scaleFactor);


#endif

protected:
  OdDbImpAssocAction *m_pImpObj;

  friend class OdDbImpAssocAction;
}; // class OdDbAssocAction

typedef OdSmartPtr<OdDbAssocAction> OdDbAssocActionPtr;
#include "TD_PackPop.h"

#endif

