/////////////////////////////////////////////////////////////////////////////// 
// Copyright (C) 2003-2011, Open Design Alliance (the "Alliance"). 
// All rights reserved. 
// 
// This software and its documentation and related materials are owned by 
// the Alliance. The software may only be incorporated into application 
// programs owned by members of the Alliance, subject to a signed 
// Membership Agreement and Supplemental Software License Agreement with the
// Alliance. The structure and organization of this software are the valuable  
// trade secrets of the Alliance and its suppliers. The software is also 
// protected by copyright law and international treaty provisions. Application  
// programs incorporating this software must include the following statement 
// with their copyright notices:
//   
//   This application incorporates Teigha(R) software pursuant to a license 
//   agreement with Open Design Alliance.
//   Teigha(R) for .dwg files Copyright (C) 2003-2011 by Open Design Alliance. 
//   All rights reserved.
//
// By use of this software, its documentation or related materials, you 
// acknowledge and accept the above terms.
//
//
// *DWG is the native and proprietary file format for AutoCAD(R) and a trademark  
// of Autodesk, Inc.  Open Design Alliance is not associated with Autodesk.
///////////////////////////////////////////////////////////////////////////////

#ifndef DbAssocDependency_INCLUDED_
#define DbAssocDependency_INCLUDED_ /* {Secret} */

#include "DbConstraintsExports.h"
#include "DbAssocGlobal.h"
#include "DbAssocAction.h"
#include "DbObject.h"
#include "DbImpAssocDependency.h"

#include "DbCompoundObjectId.h"

class OdDbAssocEvaluationCallback;

#include "TD_PackPush.h"

class DBCONSTRAINTS_EXPORT OdDbAssocDependency : public OdDbObject
{
public: 
  ODRX_DECLARE_MEMBERS(OdDbAssocDependency);

  OdDbAssocDependency(bool createImp = true);
  ~OdDbAssocDependency();

  OdDbObjectId dependencyBody() const;//
    
  OdResult setDependencyBody(OdDbObjectId dependencyBodyId);//

  OdDbAssocStatus status() const;//

  OdResult setStatus(OdDbAssocStatus newStatus, bool notifyOwningAction = true);//

  bool isReadDependency() const;//

  bool isWriteDependency() const;//

  void setIsReadDependency(bool bRead);//

  void setIsWriteDependency(bool bWrite);//

  bool isObjectStateDependent() const;//

  void setIsObjectStateDependent(bool bDependent);//

  int order() const;//

  void setOrder(int newOrder);//

  OdDbObjectId owningAction() const { return ownerId(); }

  OdResult setOwningAction(OdDbObjectId actionId);//

  OdDbObjectId dependentOnObject() const;//

  OdResult getDependentOnCompoundObject(OdDbCompoundObjectIdPtr&) const;

  bool isDependentOnCompoundObject() const;//

  OdDbObjectId prevDependencyOnObject() const;//

  OdDbObjectId nextDependencyOnObject() const;//

  bool isAttachedToObject() const { return !dependentOnObject().isNull(); }

  OdResult attachToObject(OdDbCompoundObjectId* compoundId);

  OdResult dependentOnObjectStatus() const;

  OdResult detachFromObject();

  OdResult updateDependentOnObject();

  void setDependentOnObject(OdDbCompoundObjectId* compoundId);

  static OdDbObjectId getFirstDependencyOnObject(OdDbObjectPtr pObject);

  static void getDependenciesOnObject(OdDbObjectPtr pObject, 
                                      bool readDependenciesWanted, 
                                      bool writeDependenciesWanted,
                                      OdDbObjectIdArray& dependencyIds);

  static OdResult notifyDependenciesOnObject(OdDbObjectPtr pObject,OdDbAssocStatus newStatus);

  bool isDelegatingToOwningAction() const;

  void setIsDelegatingToOwningAction(bool bDelegating);//

  bool hasCachedValue() const;

  bool isRelevantChange() const;//

  bool isDependentOnTheSameThingAs(const OdDbAssocDependency* pOtherDependency) const;

  bool isDependentOnObjectReadOnly() const;

  bool isEqualTo(const OdDbAssocDependency* pOtherDependency) const;

  bool isActionEvaluationInProgress() const;

  OdDbAssocEvaluationCallback* currentEvaluationCallback() const;

  void evaluate();

  void erased(const OdDbObject* dbObj, OdBool isErasing = kTrue);

  void modified(const OdDbObject* dbObj);

  void copied(const OdDbObject* pDbObj, const OdDbObject* pNewObj);

  OdResult dwgInFields(OdDbDwgFiler* pFiler);
  OdResult dwgOutFields(OdDbDwgFiler* pFiler) const;

  OdResult dxfInFields(OdDbDxfFiler* pFiler);
  OdResult dxfOutFields(OdDbDxfFiler* pFiler) const;

protected:
  friend class OdDbImpAssocDependency;
  OdDbImpAssocDependency *m_pImpObj;
}; // class OdDbAssocDependency

typedef OdSmartPtr<OdDbAssocDependency> OdDbAssocDependencyPtr;

#include "TD_PackPop.h"

#endif
