/////////////////////////////////////////////////////////////////////////////// 
// Copyright (C) 2003-2011, Open Design Alliance (the "Alliance"). 
// All rights reserved. 
// 
// This software and its documentation and related materials are owned by 
// the Alliance. The software may only be incorporated into application 
// programs owned by members of the Alliance, subject to a signed 
// Membership Agreement and Supplemental Software License Agreement with the
// Alliance. The structure and organization of this software are the valuable  
// trade secrets of the Alliance and its suppliers. The software is also 
// protected by copyright law and international treaty provisions. Application  
// programs incorporating this software must include the following statement 
// with their copyright notices:
//   
//   This application incorporates Teigha(R) software pursuant to a license 
//   agreement with Open Design Alliance.
//   Teigha(R) for .dwg files Copyright (C) 2003-2011 by Open Design Alliance. 
//   All rights reserved.
//
// By use of this software, its documentation or related materials, you 
// acknowledge and accept the above terms.
//
//
// *DWG is the native and proprietary file format for AutoCAD(R) and a trademark  
// of Autodesk, Inc.  Open Design Alliance is not associated with Autodesk.
///////////////////////////////////////////////////////////////////////////////

#ifndef DbAssocDependencyBody_INCLUDED_
#define DbAssocDependencyBody_INCLUDED_ /* {Secret} */

#include "DbConstraintsExports.h"
#include "DbAssocDependency.h"

#include "DbFiler.h"

#include "TD_PackPush.h"


class DBCONSTRAINTS_EXPORT OdDbAssocDependencyBody : public OdDbObject
{
public: 
  ODRX_DECLARE_MEMBERS(OdDbAssocDependencyBody);

  OdDbAssocDependencyBody();

  virtual ~OdDbAssocDependencyBody();

  OdDbObjectId parentDependency() const { return ownerId(); }

  OdDbAssocStatus status() const;

  OdResult setStatus(OdDbAssocStatus newStatus, 
                     bool notifyOwningAction = true);

  OdDbObjectId owningAction() const;

  OdDbObjectId dependentOnObject() const;

  bool isAttachedToObject() const { return !dependentOnObject().isNull(); }

  bool isActionEvaluationInProgress() const;

  OdDbAssocEvaluationCallback* currentEvaluationCallback() const;

  OdResult dwgInFields(OdDbDwgFiler* pFiler);
  OdResult dwgOutFields(OdDbDwgFiler* pFiler) const;

  OdResult dxfInFields(OdDbDxfFiler* pFiler);
  OdResult dxfOutFields(OdDbDxfFiler* pFiler) const;
public: 
    // Virtual methods that can be overridden by the derived classes
    virtual void evaluateOverride() = 0;
    virtual OdResult updateDependentOnObjectOverride()
    { return eNotImplemented; }

    virtual OdResult hasCachedValueOverride(bool& /*hasCachedVal*/) const
    { return eNotImplemented; }

    virtual OdResult isRelevantChangeOverride(bool& /*isRelevChange*/) const
    { return eNotImplemented; }

    virtual OdResult isDependentOnTheSameThingAsOverride(const OdDbAssocDependency* /*pOtherDependency*/, 
                                                         bool& /*isDependentOnSameThing*/) const
    { return eNotImplemented; }

    virtual OdResult isEqualToOverride(const OdDbAssocDependency* /*pOtherDependency*/, bool& /*isEqual*/) const
    { return eNotImplemented; }

    virtual void erasedOverride(const OdDbObject* /*pDbObj*/, bool /*isErasing*/)
    {}

    virtual void modifiedOverride(const OdDbObject* /*pDbObj*/) 
    {}

    virtual void clonedOverride(const OdDbObject* /*pDbObj*/, const OdDbObject* /*pNewObj*/)
    {}

}; // class OdDbAssocDependencyBody


typedef OdSmartPtr<OdDbAssocDependencyBody> OdDbAssocDependencyBodyPtr;

#include "TD_PackPop.h"

#endif


