/////////////////////////////////////////////////////////////////////////////// 
// Copyright (C) 2003-2011, Open Design Alliance (the "Alliance"). 
// All rights reserved. 
// 
// This software and its documentation and related materials are owned by 
// the Alliance. The software may only be incorporated into application 
// programs owned by members of the Alliance, subject to a signed 
// Membership Agreement and Supplemental Software License Agreement with the
// Alliance. The structure and organization of this software are the valuable  
// trade secrets of the Alliance and its suppliers. The software is also 
// protected by copyright law and international treaty provisions. Application  
// programs incorporating this software must include the following statement 
// with their copyright notices:
//   
//   This application incorporates Teigha(R) software pursuant to a license 
//   agreement with Open Design Alliance.
//   Teigha(R) for .dwg files Copyright (C) 2003-2011 by Open Design Alliance. 
//   All rights reserved.
//
// By use of this software, its documentation or related materials, you 
// acknowledge and accept the above terms.
//
//
// *DWG is the native and proprietary file format for AutoCAD(R) and a trademark  
// of Autodesk, Inc.  Open Design Alliance is not associated with Autodesk.
///////////////////////////////////////////////////////////////////////////////

#ifndef DbAssocDimDependencyBodyBase_INCLUDED_
#define DbAssocDimDependencyBodyBase_INCLUDED_ /* {Secret} */

#include "OdString.h"
#include "OdArray.h"
#include "DbEntity.h"
#include "OdConstrainedGeometry.h"
#include "DbAssocDependencyBody.h"

#include "Ge/GePoint3d.h"
#include "Ge/GeCurve3d.h"

#include "TD_PackPush.h"

class DBCONSTRAINTS_EXPORT OdDbSubentGeometry
{
public:
  OdDbSubentGeometry() : mSubentType(OdDb::kNullSubentType), mpCurve(NULL) {}

  OdDbSubentGeometry(const OdGePoint3d& pnt) 
      : mSubentType(OdDb::kVertexSubentType), mPoint(pnt), mpCurve(NULL) {}

  OdDbSubentGeometry(OdGeCurve3d* pCurve)
    : mSubentType(OdDb::kEdgeSubentType), mpCurve(pCurve) {}

  OdDb::SubentType type() const { return mSubentType; }

  OdGePoint3d point() const { return mPoint; }

  OdGeCurve3d* curve() const { return mpCurve; }
private:
  OdDb::SubentType mSubentType;
  OdGePoint3d      mPoint;
  OdGeCurve3d*     mpCurve;
};

class DBCONSTRAINTS_EXPORT OdDbAssocDimDependencyBodyBase : public OdDbAssocDependencyBody
{
public:
  ODRX_DECLARE_MEMBERS(OdDbAssocDimDependencyBodyBase);

  OdDbAssocDimDependencyBodyBase();
  //explicit AcDbAssocDimDependencyBodyBase(AcDbAssocCreateImpObject createImpObject = kAcDbAssocCreateImpObject);

  virtual ~OdDbAssocDimDependencyBodyBase();

  virtual OdString getEntityTextOverride() const = 0;

  virtual OdResult setEntityTextOverride(const OdString& newText) = 0;

  virtual double getEntityMeasurementOverride() const = 0;

  virtual bool isEntityAttachmentChangedOverride() const = 0;

  virtual OdResult updateDependentOnObjectOverride();

  class OdExplicitConstraint* constraint() const;

  OdDbObjectId variable() const; // of AcDbAssocVariable

  OdResult getConstrainedGeoms(OdArray<OdConstrainedGeometry*>& geoms) const;

  OdResult getConstrainedGeoms(OdArray<OdDbFullSubentPath>& geoms) const;

  OdResult getConstrainedGeoms(OdArray<OdDbSubentGeometry>& geoms,
                                        OdGeVector3d&                distanceDirection) const;

  OdResult getVariableNameAndExpression(OdString& name, OdString& expression, OdString& value) const;

  OdResult getEntityNameAndExpression(OdString& name, OdString& expression) const;

  OdResult setVariableNameAndExpression(const OdString& name, const OdString& expression);

  OdResult setEntityNameAndExpression(const OdString& name, const OdString& expression, const OdString& value);

  OdResult setNameAndExpression(const OdString& name, const OdString& expression); // Both variable and entity

  OdResult validateEntityText(const OdString& entityTextToValidate, 
                                       OdString&       errorMessage) const;

  OdString composeEntityText(int requiredNameFormat = -1) const;

  OdResult setVariableValueToMeasuredValue() const;

  OdResult deactivateConstraint() const;

  OdResult reactivateConstraint() const;

  bool isConstraintActive() const; // Has DCM d_node or r_node

  OdResult getMeasuredValue(double& measurement) const;

  OdResult setIsReferenceOnly(bool yesNo);

  static OdResult 
  getNameAndExpressionFromEntityText(const OdString& entityText,
                                     bool            useMeasurementIfNoText,
                                     double          measurement,
                                     bool            isAngular,
                                     OdString&       name, 
                                     OdString&       expression);

  int getCurrentlyUsedEntityNameFormat() const;

  void dragStatus(const OdDb::DragStat status);

  static OdResult getFromEntity(const OdDbObjectId& entityId, OdDbObjectId& dimDepBodyId);

  OdResult entityAttachmentPointMoved(const OdArray<OdDbSubentGeometry>& newAttachedGeometries,
                                               double measurement = 0.0);

  virtual OdResult isRelevantChangeOverride(bool& isRelevChange) const;

  virtual void evaluateOverride();

  virtual void modifiedOverride(const OdDbObject* pDbObj); 

  virtual void erasedOverride(const OdDbObject* pDbObj, bool isErasing);

  virtual OdResult subErase(bool erasing);

  static OdString formatToCurrentPrecision(const OdString& expression, bool isAngular);

  static bool setEraseDimensionIfDependencyIsErased(bool yesNo);

  static bool getEraseDimensionIfDependencyIsErased();

  class DBCONSTRAINTS_EXPORT NotificationIgnorer
  {
  public:
    NotificationIgnorer();

    ~NotificationIgnorer();

    static bool isIgnoringNotifications();

  private:
    const bool mPrevIsIgnoringNotifications;
    static bool smIsIgnoringNotifications;

  };

  OdResult dwgInFields(OdDbDwgFiler* pFiler);
  OdResult dwgOutFields(OdDbDwgFiler* pFiler) const;

  OdResult dxfInFields(OdDbDxfFiler* pFiler);
  OdResult dxfOutFields(OdDbDxfFiler* pFiler) const;
  private:
    OdString m_text;
    static bool sbEraseDimensionIfDependencyIsErased;
};

typedef OdSmartPtr<OdDbAssocDimDependencyBodyBase> OdDbAssocDimDependencyBodyBasePtr;

enum OdDbAssocConstraintType
{
    kNoneAssocConstraintType = 0,
    kDistanceAssocConstraintType,
    kHorizontalDistanceAssocConstraintType,
    kVerticalDistanceAssocConstraintType,
    kAngle0AssocConstraintType, 
    kAngle1AssocConstraintType,
    kAngle2AssocConstraintType,
    kAngle3AssocConstraintType,
    kRadiusAssocConstraintType,
    kDiameterAssocConstraintType,
};

#include "TD_PackPop.h"

#endif

