/////////////////////////////////////////////////////////////////////////////// 
// Copyright (C) 2003-2011, Open Design Alliance (the "Alliance"). 
// All rights reserved. 
// 
// This software and its documentation and related materials are owned by 
// the Alliance. The software may only be incorporated into application 
// programs owned by members of the Alliance, subject to a signed 
// Membership Agreement and Supplemental Software License Agreement with the
// Alliance. The structure and organization of this software are the valuable  
// trade secrets of the Alliance and its suppliers. The software is also 
// protected by copyright law and international treaty provisions. Application  
// programs incorporating this software must include the following statement 
// with their copyright notices:
//   
//   This application incorporates Teigha(R) software pursuant to a license 
//   agreement with Open Design Alliance.
//   Teigha(R) for .dwg files Copyright (C) 2003-2011 by Open Design Alliance. 
//   All rights reserved.
//
// By use of this software, its documentation or related materials, you 
// acknowledge and accept the above terms.
//
//
// *DWG is the native and proprietary file format for AutoCAD(R) and a trademark  
// of Autodesk, Inc.  Open Design Alliance is not associated with Autodesk.
///////////////////////////////////////////////////////////////////////////////

#ifndef OdGeomConstraint_INCLUDED
#define OdGeomConstraint_INCLUDED

#include "OdConstraintGroupNode.h"


#include "TD_PackPush.h"

// Forward class declarations.
//
class OdConstrainedGeometry;
class OdHelpParameter;
class OdEqualHelpParameterConstraint;
class OdCompositeConstraint;

class DBCONSTRAINTS_EXPORT OdGeomConstraint : public OdConstraintGroupNode
{
public:
  ODRX_DECLARE_MEMBERS(OdGeomConstraint);

  enum GeomConstraintType
  {
    kHorizontal    = 0,
    kVertical         ,
    kParallel         ,
    kPerpendicular    ,
    kNormal           ,
    kColinear         ,
    kCoincident       ,
    kConcentric       ,
    kTangent          ,
    kEqualRadius      ,
    kEqualLength      ,
    kSymmetric		    ,
    kG2Smooth         ,
    kFix			         
  };


  virtual OdResult getConnectedGeometries(OdArray<OdConstrainedGeometry*>& apConsGeoms) const;

  virtual OdResult getConnectedHelpParameters(OdArray<OdHelpParameter*>& apHelpParameters) const;

  virtual OdHelpParameter*  getConnectedHelpParameterFor(const OdConstrainedGeometry* pConsGeom) const;

  OdCompositeConstraint*    getOwningCompositeConstraint() const;

  OdResult deactivate();
  OdResult reactivate();

  bool isActive() const;

  bool isImplied() const;

  bool isInternal() const;

  bool isEnabled() const;

protected:
  OdGeomConstraint();
};

typedef OdSmartPtr<OdGeomConstraint> OdGeomConstraintPtr;


class DBCONSTRAINTS_EXPORT OdHelpParameter : public OdConstraintGroupNode
{
public:
  ODRX_DECLARE_MEMBERS(OdHelpParameter);

  OdHelpParameter();

  OdResult getConnectedGeometry(OdConstrainedGeometry*& pGeometry) const;

  OdResult getConnectedConstraint(OdGeomConstraint*& pConstraint) const;

  OdResult getConnectedEqualparamConstraints(OdArray<OdEqualHelpParameterConstraint*>& apEqualParamConstrs) const;

  double getValue() const;

  void setValue(const double &val);
};

class DBCONSTRAINTS_EXPORT OdCompositeConstraint : public OdGeomConstraint
{
public:
  ODRX_DECLARE_MEMBERS(OdCompositeConstraint);

  OdResult getOwnedConstraints(OdArray<OdGeomConstraint*>& apSubConstraints) const;

protected:
  OdCompositeConstraint();
};

typedef OdSmartPtr<OdHelpParameter> OdHelpParameterPtr;


#include "TD_PackPop.h"

#endif
