/////////////////////////////////////////////////////////////////////////////// 
// Copyright (C) 2003-2012, Open Design Alliance (the "Alliance"). 
// All rights reserved. 
// 
// This software and its documentation and related materials are owned by 
// the Alliance. The software may only be incorporated into application 
// programs owned by members of the Alliance, subject to a signed 
// Membership Agreement and Supplemental Software License Agreement with the
// Alliance. The structure and organization of this software are the valuable  
// trade secrets of the Alliance and its suppliers. The software is also 
// protected by copyright law and international treaty provisions. Application  
// programs incorporating this software must include the following statement 
// with their copyright notices:
//   
//   This application incorporates Teigha(R) software pursuant to a license 
//   agreement with Open Design Alliance.
//   Teigha(R) Copyright (C) 2003-2012 by Open Design Alliance. 
//   All rights reserved.
//
// By use of this software, its documentation or related materials, you 
// acknowledge and accept the above terms.
///////////////////////////////////////////////////////////////////////////////




#ifndef _ODA_DEFS_
#define _ODA_DEFS_

#ifndef _ODA_COMMON_INCLUDED_
#error  "Each cpp module MUST include OdaCommon.h as first header included"
#endif

#include <stdio.h>
#include <string.h>
#include <math.h>


/** Description:

    {group:TD_Namespaces}
*/
namespace OdDb
{
  enum DwgVersion
  {
    kDHL_MC0_0   = 0,
    kDHL_AC1_2   = 1,
    kDHL_AC1_40  = 2,
    kDHL_AC1_50  = 3,
    kDHL_AC2_20  = 4,
    kDHL_AC2_10  = 5,
    kDHL_AC2_21  = 6,
    kDHL_AC2_22  = 7,
    kDHL_1001    = 8,
    kDHL_1002    = 9,             // Release 2.5
    vAC025       = kDHL_1002,     // Release 2.5
    kDHL_1003    = 10,            // Release 2.6
    vAC026       = kDHL_1003,     // Release 2.6
    kDHL_1004    = 11,            // Release 9
    vAC09        = kDHL_1004,     // Release 9
    kDHL_1005    = 12,
    kDHL_1006    = 13,            // Release 10
    vAC10        = kDHL_1006,     // Release 10
    kDHL_1007    = 14,
    kDHL_1008    = 15,
    kDHL_1009    = 16,             // R11 and R12
    vAC12        = kDHL_1009,      // R11 and R12
    kDHL_1010    = 17,
    kDHL_1011    = 18,
    kDHL_1012    = 19,             // R13
    vAC13        = kDHL_1012,      // R13
    kDHL_1013    = 20,             // R14 beta
    vAC14beta    = kDHL_1013,      // R14 beta
    kDHL_1014    = 21,             // R14 release
    vAC14        = kDHL_1014,      // R14 release
    kDHL_1500    = 22,             // R15 (2000) beta
    kDHL_1015    = 23,             // R15 (2000) release
    vAC15        = kDHL_1015,      // R15 (2000) release
    kDHL_1800a   = 24,             // R18 (2004) beta
    kDHL_1800    = 25,             // R18 (2004) release
    vAC18        = kDHL_1800,      // R18 (2004) release
    kDHL_2100a   = 26,             // R21 (2007) beta
    kDHL_1021    = 27,             // R21 (2007) release
    vAC21        = kDHL_1021,      // R21 (2007) release
    kDHL_2400a   = 28,             // R24 (2010) beta
    kDHL_1024    = 29,             // R24 (2010) release
    vAC24        = kDHL_1024,      // R24 (2010) release
    kDHL_2700a   = 30,             // R27 (2013) beta
    kDHL_1027    = 31,             // R27 (2013) release
    vAC27        = kDHL_1027,      // R27 (2013) release

#ifdef ODA_AC27_SUPPORT
    kDHL_PRECURR = vAC24,
    kDHL_CURRENT = vAC27,
#else
    kDHL_PRECURR = vAC21,
    kDHL_CURRENT = vAC24,
#endif
    kDHL_Unknown = 32766,
    kDHL_Max     = 32767
  };

  enum MaintReleaseVer
  {
    kMRelease0 = 0,
    kMRelease1,
    kMRelease2,
    kMRelease3,
    kMRelease4,
    kMRelease5,
    kMRelease6,
    kMRelease7,
    kMRelease8,
    kMRelease9,
    kMRelease10,
    kMRelease11,
    kMRelease12,
    kMRelease13,
    kMRelease14,
    kMRelease15,
    kMRelease16,
    kMRelease17,
    kMRelease18,
    kMRelease19,
    kMRelease20,
    kMRelease21,
    kMRelease22,
    kMRelease23,
    kMRelease24,
    kMRelease25,
    kMRelease26,
    kMRelease27,
    kMRelease28,
    kMRelease29,
    kMRelease30,
    kMRelease31,
    kMRelease32,
    kMRelease33,
    kMRelease34,
    kMRelease35,
    kMRelease36,
    kMRelease37,
    kMRelease38,
    kMRelease39,
    kMRelease40,
    kMRelease41,
    kMRelease42,
    kMRelease43,
    kMRelease44,
    kMRelease45,
    kMRelease46,
    kMRelease47,
    kMRelease48,
    kMRelease49,
    kMReleaseFirstValid1500 = kMRelease41,
#ifdef ODA_AC27_SUPPORT
    kMReleaseCurrent = kMRelease8,
#else
    kMReleaseCurrent = kMRelease6,
#endif
    kMReleaseUnknown = 126,
    kMReleaseMax = 127
  };

  // lineweights are in 100ths of a millimeter
  enum LineWeight
  {
    kLnWt000          =   0,
    kLnWt005          =   5,
    kLnWt009          =   9,
    kLnWt013          =  13,
    kLnWt015          =  15,
    kLnWt018          =  18,
    kLnWt020          =  20,
    kLnWt025          =  25,
    kLnWt030          =  30,
    kLnWt035          =  35,
    kLnWt040          =  40,
    kLnWt050          =  50,
    kLnWt053          =  53,
    kLnWt060          =  60,
    kLnWt070          =  70,
    kLnWt080          =  80,
    kLnWt090          =  90,
    kLnWt100          = 100,
    kLnWt106          = 106,
    kLnWt120          = 120,
    kLnWt140          = 140,
    kLnWt158          = 158,
    kLnWt200          = 200,
    kLnWt211          = 211,
    kLnWtByLayer      =  -1,
    kLnWtByBlock      =  -2,
    kLnWtByLwDefault  =  -3
  };

  enum PlotStyleNameType
  {
    kPlotStyleNameByLayer       = 0,
    kPlotStyleNameByBlock       = 1,
    kPlotStyleNameIsDictDefault = 2,
    kPlotStyleNameById          = 3
  };

  // Do not append any enums here !
}

#define SETBIT(flags, bit, value) ((value) ? (flags |= (bit)) : (flags &= ~(bit)))
#define GETBIT(flags, bit) (((flags) & (bit)) ? true : false)

#define SETBIT_0(flags, bit) (flags &= ~(bit))
#define SETBIT_1(flags, bit) (flags |=  (bit))

inline bool OdPositive(double x, double tol = 1.e-10)
{
  return (x > tol);
}

inline bool OdNegative(double x, double tol = 1.e-10)
{
  return (x < -tol);
}

inline bool OdZero(double x, double tol = 1.e-10)
{
  return !OdPositive(x, tol) && !OdNegative(x, tol);
}

inline bool OdNonZero(double x, double tol = 1.e-10)
{
  return OdPositive(x, tol) || OdNegative(x, tol);
}

inline bool OdEqual(double x, double y, double tol = 1.e-10)
{
  double diff = x - y;
  if(!OdZero(diff, tol))
  {
    if(tol < 1.e-7)
    {
      double absX = fabs(x);
      double absY = fabs(y);
      double maxVal = absX > absY ? absX : absY;
      diff /= maxVal;
      return OdZero(diff, tol);
    }
    return false;
  }

  return true;

}

inline bool OdLess(double x, double y, double tol = 1.e-10)
{
  return x < y - tol;
}

inline bool OdLessOrEqual(double x, double y, double tol = 1.e-10)
{
  return x <= y + tol;
}

inline bool OdGreater(double x, double y, double tol = 1.e-10)
{
  return x > y + tol;
}

inline bool OdGreaterOrEqual(double x, double y, double tol = 1.e-10)
{
  return x >= y - tol;
}

inline double OdSign(double x)
{
  return (x < 0.0 ? -1.0 : (x > 0.0 ? 1.0 : 0.0));
}

inline int OdSign(int x)
{
  return (x < 0 ? -1 : (x > 0 ? 1 : 0));
}

inline int OdCmpDouble(double x, double y, double tol = 1.e-10)
{
  if (OdEqual(x,y,tol))
    return 0;
  if (OdPositive(x - y, tol))
    return 1;
  else
    return -1;
}

inline double safeDivide( double a, double b)
{
  if (a == 0.)
    return 0.;
  
  if (b != 0.)
  {
    // Binary exponent of 1.e+300 (value which can be safely represented
    // by IEEE double. Note that even if on some platforms double can represent
    // larger values they are saved to a .dwg file as IEEE where
    // DBL_MAX = 1.7976931348623158e+308 and DBL_MAX_EXP = 1024
    static const int maxBinExp = 997;
    int nExpA, nExpB;
    //double dMantissa = 
    frexp(a, &nExpA);
    //dMantissa = 
    frexp(b, &nExpB);
    if ((nExpA - nExpB) < maxBinExp)
      return a / b;
  }

  bool bNeg = (a < 0.);
  if (b < 0.)
    bNeg = !bNeg;

  return bNeg ? -1.e+300 : 1.e+300;
}



#endif // _ODA_DEFS_

